function [A,S,delta] = WHAM(Ctot, n, J, beta, Atol, maxIter)
%This function uses the WHAM algorithm to stitch together histograms taken
%over a range of temperatures.
%   The inputs are:
%       Ctot - a structre with the counts for the entire range of the histograms 
%       n - the number of counts per temperature
%       J - the number of temperatures
%       beta - the inverse of each temperature
%       Atol - the tolerance to stop iterating and return A and S
%       maxIter - the maximum number of iterations afterwhich the system
%       returns a warning and A,S
%   The outputs are:
%       A(j) - Helmoltz free energy of the system at a given temperature
%       S(u) - The entropy of the system at a given free energy

logn = log(n);

for j = 1:J
    betajU(j).bins = beta(j) * Ctot.bins;
end
clear j

A_old = zeros(J,1);
S = zeros(length(Ctot.bins),1);
iter = 1;

for j = 1:J
    betajA_old(j) = beta(j) .* A_old(j);
end

clear j

% This belongs inside a loop over U

for U = 1:length(Ctot.bins)
    sumj(U) = 0;
    for j = 1:length(betajU)
        sumj(U) = sumj(U) + exp(-betajU(j).bins(U) + betajA_old(j));
    end
    S(U) = log(Ctot.counts(U)) - logn - log(sumj(U));
end



clear i U j 

for j = 1:J
    sumU(j) = 0;
    for U = 1:length(S)
        sumU(j) = sumU(j) + exp(S(U) - betajU(j).bins(U));
    end
    betajA_new(j) = -1 .* log(sumU(j));
end


delta = (norm(betajA_old-betajA_new))/norm(norm(betajA_new));

betajA_old = betajA_new;

while delta > Atol
    betajA_old = betajA_new;
    
    
    
    for U = 1:length(Ctot.bins)
        sumj(U) = 0;
        for j = 1:length(betajU)
            sumj(U) = sumj(U) + exp(-betajU(j).bins(U) + betajA_old(j));
        end
        S(U) = log(Ctot.counts(U)) - logn - log(sumj(U));
    end

    clear U j 

    for j = 1:J
        sumU(j) = 0;
        for U = 1:length(S)
            sumU(j) = sumU(j) + exp(S(U) - betajU(j).bins(U));
        end
        betajA_new(j) = -1 .* log(sumU(j));
    end
    
    clear U j
    
    iter = iter + 1;
    
    if iter > maxIter
        for j = 1:J
            minbetajA_old = min(betajA_old);
            betajA_old = betajA_old - minbetajA_old;
            A(j) = betajA_old(j) ./ beta(j);
        end
        disp('returned after maximum iterations')
        disp(num2str(iter))
      return
    end
    
    delta(iter) = (norm(betajA_old-betajA_new))/norm(norm(betajA_new));
    
    minbetajA_old = min(betajA_old);
    betajA_old = betajA_old - minbetajA_old;
       
end

for j = 1:J
    A(j) = betajA_old(j) ./ beta(j);
end

end

